function isDataset(obj) {
    return obj.constructor === Dataset;
}
export function createDataset(cg, data) {
    if (isDataset(data)) {
        return data;
    }
    return new Dataset(cg, data);
}
export class Dataset {
    constructor(cg, data) {
        this.length = 0;
        this.disposed = false;
        this.update = (data) => {
            if (this.disposed) {
                throw new Error("This DataSet cannot be updated, it's already been disposed.");
            }
            if (typeof data === "number") {
                this.buffer([data]);
                this.length = 1;
            }
            else if (ArrayBuffer.isView(data)) {
                this.buffer(data);
                this.length = data.length;
            }
            else if (Array.isArray(data)) {
                this.buffer(data);
                if (typeof data[0] === "number") {
                    this.length = data.length;
                }
                else {
                    this.length = data.length * data[0].length;
                }
            }
            return this;
        };
        this.buffer = cg.regl.buffer(1);
        this.update(data);
    }
    dispose() {
        this.buffer.destroy();
        this.disposed = true;
    }
    count(size) {
        this.assertFits(size);
        return this.length / size;
    }
    divisor(instances, size) {
        const count = this.count(size);
        if (count === 1) {
            return instances;
        }
        return 1;
    }
    assertFits(size) {
        if (this.length % size !== 0) {
            throw new Error(`Attempted to use incompatible size ${size} with data of length ${this.length}.`);
        }
    }
}
//# sourceMappingURL=dataset.js.map