import { Primitive } from "../common";
import { createDataset } from "./dataset";
const DEFAULT_OPTIONS = {
    color: [0, 0, 0, 0.5],
};
export class Triangles extends Primitive {
    constructor(cg, vertices, options = {}) {
        super();
        this.cg = cg;
        const opts = Object.assign(Object.assign({}, DEFAULT_OPTIONS), options);
        this.vertices = createDataset(cg, vertices);
        this.color = opts.color.slice();
    }
    command(glsl) {
        return this.cg.regl({
            vert: `
          precision highp float;
          attribute vec2 position;

          ${glsl}

          void main() {
            gl_Position = domainToClip(position);
          }`,
            frag: `
          precision highp float;
          uniform vec4 color;
          void main() {
            gl_FragColor = color;
          }`,
            attributes: {
                position: this.cg.regl.prop("position"),
            },
            uniforms: {
                color: this.cg.regl.prop("color"),
            },
            count: this.cg.regl.prop("count"),
        });
    }
    render(command) {
        const { vertices, color } = this;
        command({
            position: vertices.buffer,
            count: vertices.count(2),
            color,
        });
    }
    dispose() {
        this.vertices.dispose();
    }
}
//# sourceMappingURL=triangles.js.map